local Vector = require("libs.hump.vector")
local Entity = Class("Entity")

function Entity:init()
	-- Position
	self.position = Vector.new(0, 0)

	-- Speed
	self.baseSpeed = 0
	self.speed = Vector.new(0, 0)

	-- Look
	self.lookSpeed = 0
	self.look = 0

	-- Rotation
	self.rotation = 0

	-- World
	self.map = nil
	self.id = 0

	-- Hitbox
	self.radius = 0
	self.collidesWith = {}

	-- Alive?
	self.alive = true

	-- ECS
	self.components = {}
end

-- ECS functions
function Entity:has(component)
	return (self.components[component] ~= nil)
end

function Entity:attach(name, component)
	component.entity = self
	if(component.onAttach) then component:onAttach() end
	self.components[name] = component
end

function Entity:detach(name)
	if(not self.components[name]) then return end
	local component = self.components[name]
	if(component.onDetach) then component:onDetach() end
	self.components[name] = nil
end

function Entity:get(name)
	return self.components[name]
end

function Entity:update(dt)
	for _, component in pairs(self.components) do
		if(component.update ~= nil) then
			component:update(dt)
		end
	end
end

function Entity:draw(debug)
	for _, component in pairs(self.components) do
		if(component.draw ~= nil) then
			component:draw(debug)
		end
	end
end
-- End ECS functions

return Entity
