local Vector = require("libs.hump.vector")
local Movement = require("entities.parts.movement")
local Sprite = require("entities.parts.sprite")
local Explode = require("entities.parts.explode")
local Death = require("entities.parts.death")
local AnimFlow = require("entities.parts.animflow")
local AnimDeath = require("entities.parts.animdeath")
local AnimFacing = require("entities.parts.animfacing")
local Entity = require("entities.entity")
local Explosion = Entity:extend("Explosion")

local super = Explosion.super
local image = load:image("etc/explosion.png", {255, 0, 255})
local g = Anim8.newGrid(32, 32, image:getDimensions())

function Explosion:init()
	super.init(self)

	-- Position
	self.position = Vector.new(0, 0)

	-- Speed
	self.baseSpeed = 0
	self.speed = Vector.new(0, 0)

	-- Look
	self.lookSpeed = 0
	self.look = 0

	-- Rotation
	self.rotation = 0

	-- Animation
	self.spriteData = {}
	self.spriteOffset = Vector.new(0, 5)
	self.faces = {
		center = {}
	}
	self.image = image
	self.animation = nil

	-- Hitbox
	self.collidesWith = {}

	-- World
	self.map = nil
	self.id = 0
	self.minimapIcon = {255, 32, 32}

	-- Alive?
	self.alive = true

	-- Components
	self:attach("movement", Movement:new())
	self:attach("sprite", Sprite:new())
	self:attach("explode", Explode:new())
	self:attach("animflow", AnimFlow:new())
	self:attach("animdeath", AnimDeath:new())
	self:attach("animfacing", AnimFacing:new())
	self:attach("death", Death:new())

	-- Animation array
	local f = { "left", "right" }
	local t = { "mid", "end" }
	for i, faces in ipairs(f) do
		self.faces[faces] = {}
		for j, eTypes in ipairs(t) do
			j = j - 1
			self.faces[faces][eTypes] = Anim8.newAnimation(g("1-5", (i*2)-j, "4-1", (i*2)-j), 0.15)
		end
	end
	self.faces["center"]["center"] = Anim8.newAnimation(g("1-5", 9, "4-1", 9), 0.15)
	self.faces["left"]["end"] = self.faces["left"]["mid"]
	self.faces["right"]["end"] = self.faces["right"]["mid"]
end

function Explosion:setType(eType)
	self.type = eType
	if(eType == "center") then self:face(eType) end
end

return Explosion
