local Bomb = require("entities.bomb")
local BombGhost = require("entities.bombghost")
local Component = require("entities.component")
local BombPlace = Component:extend("Bomb Place Component")

local floor, cos, sin = math.floor, math.cos, math.sin

function BombPlace:onAttach()
	local entity = self.entity

	entity.setGhost = self.setGhost
	entity.setBomb = self.setBomb
end

function BombPlace:onDetach()
	local entity = self.entity

	entity.setGhost = nil
	entity.setBomb = nil
end

function BombPlace:setGhost()
	local x = floor(self.position.x + cos(self.rotation))
	local y = floor(self.position.y + sin(self.rotation))
	local map = self.map

	if(self.ghost == nil) then
		local ghost = BombGhost:new()
		ghost.owner = self
		ghost.colorize = self.minimapIcon
		ghost.colorize[4] = 128
		ghost:setPosition(x + 0.5, y + 0.5)
		self.ghost = ghost

		map:spawn(ghost)
	end
end

function BombPlace:setBomb()
	local x = floor(self.position.x + cos(self.rotation))
	local y = floor(self.position.y + sin(self.rotation))
	local map = self.map
	local entities = map:getEntitiesAt(x, y)
	local hasBomb = false

	if(self.ghost) then
		self.ghost.alive = false
		self.ghost = nil

		if(map:getEntityCount(x, y) > 0) then
			for _, e in pairs(entities) do
				if(e.class.name == "Bomb") then
					hasBomb = true
					break
				end
			end
		end

		if(not hasBomb) then
			local bomb = Bomb:new()
			bomb.owner = self
			bomb.power = self.bombPower
			bomb.minimapIcon = self.minimapIcon
			bomb:setPosition(x + 0.5, y + 0.5)

			map:spawn(bomb)
		end
	end
end

return BombPlace
