local SoftWallDeath = require("entities.softwalldeath")
local Component = require("entities.component")
local Explode = Component:extend("Explode Component")

local rad, floor, cos = math.rad, math.floor, math.cos
local twoPi = math.pi * 2

function Explode:init()
	self.doOnce = true
end

function Explode:update(dt)
	local entity = self.entity
	local map = entity.map
	local layer = map.layer[map.raycastLayer]
	local x = floor(entity.position.x)
	local y = floor(entity.position.y)

	-- Only do once, instead of repeating until explosion dissipates
	if(self.doOnce) then
		local tiles = {
			right	= {x = x + 1, y = y},
			down	= {x = x, y = y + 1},
			left	= {x = x - 1, y = y},
			up		= {x = x, y = y - 1}
		}
		local tilesToCheck = {}
		-- Destroy walls in front of explosion except when its the end
		if(entity.rotation == 0) then
			tilesToCheck[#tilesToCheck+1] = tiles.right
		end
		if(entity.type == "center" or entity.rotation == twoPi * 0.25) then
			tilesToCheck[#tilesToCheck+1] = tiles.down
		end
		if(entity.type == "center" or entity.rotation == twoPi * 0.5) then
			tilesToCheck[#tilesToCheck+1] = tiles.left
		end
		if(entity.type == "center" or entity.rotation == twoPi * 0.75) then
			tilesToCheck[#tilesToCheck+1] = tiles.up
		end
		if(entity.type ~= "end") then
			for _, tile in ipairs(tilesToCheck) do
				if(not map:isOutOfBounds(tile.x, tile.y) and
				   map:hasAttribute(tile.x, tile.y, "breakable")) then
					local currentTile = layer.tiles[tile.x][tile.y]
					currentTile.char = " "
					map:clearAttributes(tile.x, tile.y)

					-- When destroying walls, spawn 4 soft wall deaths
					local edges = {
						{x = tile.x + 0.5, y = tile.y	   , r = 270},
						{x = tile.x + 1  , y = tile.y + 0.5, r = 0  },
						{x = tile.x + 0.5, y = tile.y + 1  , r = 90 },
						{x = tile.x		 , y = tile.y + 0.5, r = 180}
					}
					local entities = {}
					for _, edge in ipairs(edges) do
						local crumble = SoftWallDeath:new()
						crumble.rotation = rad(edge.r)
						crumble:setPosition(edge.x, edge.y)
						entities[#entities+1] = crumble
					end
					map:spawnAll(entities)
				end
			end
		end
		self.doOnce = false
	end

	-- Kill things in tile
	if(map:getEntityCount(x, y) > 0) then
		local entities = map:getEntitiesAt(x, y)
		for id, e in pairs(entities) do
			if(e.id ~= entity.id and
			   e.class.name ~= "Explosion") then
				e.alive = false
			end
		end
	end
end

return Explode
