local Explosion = require("entities.explosion")
local Component = require("entities.component")
local Fuse = Component:extend("Fuse Component")

function Fuse:onAttach()
	local entity = self.entity

	entity.onDeath = self.explode
end

function Fuse:onDetach()
	local entity = self.entity

	entity.onDeath = nil
end

function Fuse:update(dt)
	local entity = self.entity
	entity.fuse = entity.fuse - dt
	if(entity.fuse <= 0) then
		entity.alive = false
	end
end

local function createMid(map, x, y, type)
	local layer = map.layer[map.raycastLayer]

	if(not map:isBlocking(x, y)) then
		local explosion = Explosion:new()
		explosion:setType("center")
		explosion:setPosition(x, y)

		map:spawn(explosion)
	end
end

local function createLine(map, x, y, dx, dy, power, interval, initTime)
	local layer = map.layer[map.raycastLayer]
	local entities = {}
	local currentInterval = initTime or 0
	interval = interval or 0

	local dxR, dyR = {
		[-1] = { angle = 180, face = "left" },
		[1] = { angle = 0, face = "right" }
	}, {
		[-1] = { angle = 90, face = "right" },
		[1] = { angle = 270, face = "left" }
	}

	for i=1, power do
		local m, n = x+(i*dx), y+(i*dy)
		if(not map:isBlocking(m, n)) then
			local explosion = Explosion:new()
			local angle = (dxR[dx]) and dxR[dx].angle or dyR[dy].angle
			explosion.rotation = math.rad(angle)
			explosion:setType((i ~= power) and "mid" or "end")
			explosion:face((dxR[dx]) and dxR[dx].face or dyR[dy].face)
			explosion:setPosition(m, n)
			entities[#entities+1] = explosion
			entities[#entities+1] = currentInterval
			currentInterval = currentInterval + interval
		else
			break
		end
	end

	map:spawnAll(entities)
end

function Fuse:explode()
	local map = self.map
	local x = math.floor(self.position.x) + 0.5
	local y = math.floor(self.position.y) + 0.5

	-- Check x amount of squares from bomb
	-- Where x is bomb power
	createMid(map, x, y)
	createLine(map, x, y, 1, 0, self.power)
	createLine(map, x, y, 0, 1, self.power)
	createLine(map, x, y, -1, 0, self.power)
	createLine(map, x, y, 0, -1, self.power)
end

return Fuse
