local Vector = require("libs.hump.vector")
local Component = require("entities.component")
local Movement = Component:extend("Movement Component")

local sin, cos, atan2 = math.sin, math.cos, math.atan2

function Movement:onAttach()
	local entity = self.entity

	entity.setPosition = self.setPosition
end

function Movement:onDetach()
	local entity = self.entity

	entity.setPosition = nil
end

function Movement:update(dt)
	local entity = self.entity
	local map = entity.map

	entity.speed:normalize_inplace()
	local moveStepX = entity.speed.x * entity.baseSpeed * dt
	local moveStepY = entity.speed.y * entity.baseSpeed * dt
	local ox, oy = entity.position:unpack()
	local nx, ny = ox, oy

	local x = entity.position.x +
			  (cos(entity.rotation) * moveStepY) -
			  (sin(entity.rotation) * moveStepX)
	local y = entity.position.y +
			  (sin(entity.rotation) * moveStepY) +
			  (cos(entity.rotation) * moveStepX)

	local ax, ay = ox, oy
	local bx, by = ox, oy
	if(x > ox) then
		ax = x + entity.radius
		bx = ax
		ay = oy - entity.radius
		by = oy + entity.radius
	elseif(x < ox) then
		ax = x - entity.radius
		bx = ax
		ay = oy - entity.radius
		by = oy + entity.radius
	end
	if(not map:isBlocking(ax, ay) and
	   not map:isBlocking(bx, by)) then
		nx = x
	end

	if(y > oy) then
		ax = ox - entity.radius
		bx = ox + entity.radius
		ay = y + entity.radius
		by = ay
	elseif(y < oy) then
		ax = ox - entity.radius
		bx = ox + entity.radius
		ay = y - entity.radius
		by = ay
	end
	if(not map:isBlocking(ax, ay) and
	   not map:isBlocking(bx, by)) then
		ny = y
	end

	for _, e in ipairs(map.entities) do
		local collides = entity.collidesWith[e.class.name]

		if entity.id ~= e.id and collides then
			local newPos = Vector.new(nx, ny)
			local delta = newPos - e.position
			local dist = delta:len()
			local closest = entity.radius + e.radius
			if(dist < closest and dist >= e.radius) then
				local angle = atan2(delta.y, delta.x)
				nx = e.position.x + cos(angle) * closest
				ny = e.position.y + sin(angle) * closest
			end
		end
	end

	entity.position.x = nx
	entity.position.y = ny
end

function Movement:setPosition(x, y)
	self.position.x = x or 0
	self.position.y = y or 0
end

return Movement
