local Vector = require("libs.hump.vector")
local Movement = require("entities.parts.movement")
local Sprite = require("entities.parts.sprite")
local AnimDeath = require("entities.parts.animdeath")
local AnimFlow = require("entities.parts.animflow")
local AnimFacing = require("entities.parts.animfacing")
local Death = require("entities.parts.death")
local Entity = require("entities.entity")
local SoftWallDeath = Entity:extend("Soft Wall Death")

local super = SoftWallDeath.super
local image = load:image("etc/soft_wall.png", { 255, 0, 255 })
local g = Anim8.newGrid(32, 32, image:getDimensions())

function SoftWallDeath:init()
	super.init(self)

	-- Animation
	self.spriteData = {}
	self.spriteOffset = Vector.new(0, 0)
	self.faces = {}
	self.image = image
	self.animation = nil

	-- Components
	self:attach("movement", Movement:new())
	self:attach("sprite", Sprite:new())
	self:attach("animdeath", AnimDeath:new())
	self:attach("animflow", AnimFlow:new())
	self:attach("animfacing", AnimFacing:new())
	self:attach("death", Death:new())

	-- Initialize the animation array
	local f = { "front", "right", "back", "left" }
	for i, face in ipairs(f) do
		self.faces[face] = Anim8.newAnimation(g("1-6", i), 0.45)
	end
end

return SoftWallDeath
