local Entity = require("entities.entity")
local Spawner = Class("Spawner")

function Spawner:init(map)
	self.map = map

	-- List of entities to spawn
	-- value is a table where e = entity, t = time
	self.entities = {}
end

function Spawner:spawn(entity, time)
	if(entity == nil) then return end
	self.entities[#self.entities+1] = {
		e = entity,
		t = time or 0
	}
end

function Spawner:spawnAll(...)
	local args = { n = select("#", ...), ... }
	local argList = {}

	for i=1, args.n do
		if(type(args[i]) == "table" and
		   not Class.isInstance(args[i])) then
			for k, v in pairs(args[i]) do
				argList[#argList+1] = v
			end
		else
			argList[#argList+1] = args[i]
		end
	end

	for i=1, #argList do
		local entity = argList[i]
		local time = argList[i+1]
		if(Class.isInstance(entity) and
		   entity.class:extends(Entity)) then
			if(type(time) == "number") then
				self:spawn(entity, time)
			else
				self:spawn(entity)
			end
		end
	end
end

function Spawner:update(dt)
	-- print("Entity Count: "..#self.entities)
	for _, t in ipairs(self.entities) do
		t.t = t.t - dt

		if(t.t <= 0) then
			self.map:addEntity(t.e)
		end
	end

	local index = 1
	local size = #self.entities
	while index <= size do
		if(self.entities[index].t <= 0) then
			self.entities[index] = self.entities[size]
			self.entities[size] = nil
			size = size - 1
		else
			index = index + 1
		end
	end
end

return Spawner
