local Vector = require("libs.hump.vector")
local Movement = require("entities.parts.movement")
local Look = require("entities.parts.look")
local Sprite = require("entities.parts.sprite")
local ViewBob = require("entities.parts.viewbob")
local MoveAnim = require("entities.parts.moveanim")
local AnimFlow = require("entities.parts.animflow")
local AnimFacing = require("entities.parts.animfacing")
local BombPlace = require("entities.parts.bombplace")
local Death = require("entities.parts.death")
local DropCorpse = require("entities.parts.dropcorpse")
local Entity = require("entities.entity")
local Unit = Entity:extend("Unit")

local super = Unit.super
local sprites = {
	load:image("players/white.png", {255, 0, 255}),
	load:image("players/black.png", {255, 0, 255})
}
local icons = {
	{255, 255, 255},
	{128, 128, 128},
	{0, 128, 255},
	{255, 0, 128}
}
local g = Anim8.newGrid(32, 32, sprites[1]:getDimensions())

function Unit:init()
	super.init(self)

	-- Speed
	self.baseSpeed = 1.5

	-- Look
	self.lookSpeed = math.pi

	-- Animation
	self.spriteData = {}
	self.spriteOffset = Vector.new(0, 1)
	self.faces = {}
	self.image = nil
	self.animation = nil

	-- World
	self.minimapIcon = nil
	self.eyeHeight = 0.5

	-- Hitbox
	self.radius = 0.25
	self.collidesWith = {
		Unit = true,
		Bomb = true
	}

	-- Components
	self:attach("movement", Movement:new())
	self:attach("look", Look:new())
	self:attach("sprite", Sprite:new())
	self:attach("moveanim", MoveAnim:new())
	self:attach("animflow", AnimFlow:new())
	self:attach("animfacing", AnimFacing:new())
	self:attach("viewbob", ViewBob:new())
	self:attach("bombplace", BombPlace:new())
	self:attach("dropcorpse", DropCorpse:new())
	self:attach("death", Death:new())

	-- Bombs
	self.bombs = 1
	self.bombPower = 1

	-- Unit
	self.playerID = 0

	-- Initialize the animation array
	local f = { "front", "left", "back", "right" }
	for i, face in ipairs(f) do
		self.faces[face] = Anim8.newAnimation(g(2, i, "1-3", i), 0.175)
	end
end

function Unit:setID(id)
	if(id < 1 and id > #sprites) then return end
	self.playerID = id
	self.image = sprites[id]
	self.minimapIcon = icons[id]
end

return Unit
